package com.keao.alimeeting.detail;

import android.content.Context;
import android.support.constraint.ConstraintLayout;
import android.support.v7.util.SortedList;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.keao.alimeeting.R;
import com.keao.alimeeting.utils.Logger;
import com.keao.alimeeting.utils.Platform;
import com.keao.alimeeting.widget.GridItemDecoration;
import com.keao.alimeeting.widget.PageGridLayoutManager;
import com.aliwork.meeting.api.member.AMSDKMeetingClient;
import com.aliwork.meeting.api.render.AMSDKEglBase;
import com.keao.alimeeting.widget.UserRenderView;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import static com.keao.alimeeting.utils.PlatformKt.dipToPx;

/**
 * Created by Ivan on 16/9/30.
 */

public class RenderListAdapter extends RecyclerView.Adapter<RenderListAdapter.RenderHolder> {

    private final static String LOG_TAG = "RenderListAdapter";

    public interface RenderListener {

        void onItemClicked(int position);

        void onVisibleRenderChanged(int position, boolean visible);

        void onDataSetChanged();
    }


    private final SortedList.Callback<AMSDKMeetingClient> mSortedCallback
            = new SortedList.Callback<AMSDKMeetingClient>() {
        @Override
        public int compare(AMSDKMeetingClient o1, AMSDKMeetingClient o2) {
            if (o1.isPublisher() == o2.isPublisher()) {
                if (o1.isOnline() == o2.isOnline()) {
                    return 0;
                } else if (o1.isOnline()) {
                    return -1;
                } else {
                    return 1;
                }
            } else if (o1.isPublisher()) {
                return -1;
            } else {
                return 1;
            }

        }

        @Override
        public void onChanged(int position, int count) {
            Logger.d(LOG_TAG, "Data onChanged, position:" + position + " count:" + count);
            notifyItemRangeChanged(position, count);
            int firstPos = mPageSize * mLayoutManager.getCurrentPage();
            int lastPos = firstPos + mPageSize;
            for (int i = position; i < position + count && i >= firstPos && i < lastPos; i++) {
                mListener.onVisibleRenderChanged(position, true);
            }
        }

        @Override
        public boolean areContentsTheSame(AMSDKMeetingClient oldItem, AMSDKMeetingClient newItem) {
            return (TextUtils.equals(oldItem.getUuid(), newItem.getUuid())
                    && TextUtils.equals(oldItem.getName(), oldItem.getName())
                    && oldItem.getClientType() == newItem.getClientType()
                    && oldItem.isOnline() == newItem.isOnline()
                    && oldItem.isVideoOn() == newItem.isVideoOn()
                    && oldItem.isTalking() == newItem.isTalking()
                    && oldItem.isStreamReady() == newItem.isStreamReady());
        }

        @Override
        public boolean areItemsTheSame(AMSDKMeetingClient item1, AMSDKMeetingClient item2) {
            return TextUtils.equals(item1.getUuid(), item2.getUuid());
        }

        @Override
        public void onInserted(int position, int count) {
            int size = mUsers.size();
            int orgSize = size - count;
            if (orgSize <= 1 || (orgSize <= 4 && size > 4)) {
                Logger.d(LOG_TAG, "Data onInserted, dataSetChanged");
                notifyDataSetChanged();
                mListener.onDataSetChanged();
            } else {
                Logger.d(LOG_TAG, "Data onInserted, position:" + position + " count:" + count);
                notifyItemRangeInserted(position, count);
            }
        }

        @Override
        public void onRemoved(int position, int count) {
            int size = mUsers.size();
            int orgSize = size + count;
            if (size == 1 || (orgSize > 4 && size <= 4)) {
                Logger.d(LOG_TAG, "Data onRemoved, dataSetChanged");
                notifyDataSetChanged();
                mListener.onDataSetChanged();
            } else {
                Logger.d(LOG_TAG, "Data onRemoved, position:" + position + " count:" + count);
                notifyItemRangeRemoved(position, count);
            }
        }

        @Override
        public void onMoved(int fromPosition, int toPosition) {
            Logger.d(LOG_TAG, "Data onMoved, fromPos:" + fromPosition + " toPos:" + toPosition);
            notifyItemMoved(fromPosition, toPosition);
            //TODO: RecyclerView bug: https://code.google.com/p/android/issues/detail?id=99047
            //when move to position 0, it will have a auto scroll
            if (mLayoutManager.getCurrentPage() == 0 && (fromPosition == 0 || toPosition == 0)) {
                mLayoutManager.scrollToPosition(0);
            }
            int firstPos = mPageSize * mLayoutManager.getCurrentPage();
            int lastPos = firstPos + mPageSize;
            boolean fromPosIsVisible = fromPosition >= firstPos && fromPosition < lastPos;
            boolean toPosIsVisible = toPosition >= firstPos && toPosition < lastPos;
            if (fromPosIsVisible != toPosIsVisible) {
                mListener.onVisibleRenderChanged(toPosition, toPosIsVisible);
            }

        }
    };

    private static final int LARGE_PAGE_SPAN_COUNT = 3;
    private static final int NORMAL_PAGE_SPAN_COUNT = 2;
    private static final int SMALL_PAGE_SPAN_COUNT = 1;
    private static final int LARGE_PAGE_SIZE = LARGE_PAGE_SPAN_COUNT * LARGE_PAGE_SPAN_COUNT;
    private static final int NORMAL_PAGE_SIZE = NORMAL_PAGE_SPAN_COUNT * NORMAL_PAGE_SPAN_COUNT;
    private static final int SMALL_PAGE_SIZE = SMALL_PAGE_SPAN_COUNT * SMALL_PAGE_SPAN_COUNT;

    private final LinkedList<AMSDKMeetingClient> mUsers = new LinkedList<>();
    private final List<Boolean> mPlaceHolders = new ArrayList<>();
    private final LayoutInflater mInflater;
    private final RenderListener mListener;
    private final AMSDKEglBase mEglContext;
    private final PageGridLayoutManager mLayoutManager;
    private GridItemDecoration mItemDecoration;
    private int mPageSize;

    public RenderListAdapter(Context context, RenderListener listener, AMSDKEglBase eglContext,
            PageGridLayoutManager layoutManager, RecyclerView recyclerView) {
        mInflater = LayoutInflater.from(context);
        mListener = listener;
        mEglContext = eglContext;
        mLayoutManager = layoutManager;
        setHasStableIds(true);
        mItemDecoration = new GridItemDecoration(dipToPx(1), 2);
        recyclerView.addItemDecoration(mItemDecoration);
    }


    @Override
    public RenderHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        return new RenderHolder(mInflater.inflate(R.layout.layout_render_view, parent, false),
                mEglContext, mListener);
    }

    @Override
    public void onBindViewHolder(RenderHolder holder, int position) {
        int size = mUsers.size();
        if (position < size) {
            AMSDKMeetingClient vo = mUsers.get(position);
            holder.setUp(vo);
        }
    }



    @Override
    public int getItemCount() {
        return mUsers.size() + mPlaceHolders.size();
    }

    int calculateItemSize(int size) {
        if (size > 9 && size % 9 != 0) {
            return (size / 9 + 1) * 9;
        } else {
            return size;
        }
    }

    public int getRealUserSize() {
        return mUsers.size();
    }

    @Override
    public void onViewRecycled(RenderHolder holder) {
        if (holder != null) {
            holder.release();
        }
    }

    @Override
    public long getItemId(int position) {
        if (position < mUsers.size()) {
            return mUsers.get(position).getUuid().hashCode();
        } else {
            return 0l;
        }
    }

    public void setUsers(List<AMSDKMeetingClient> users) {
        mUsers.clear();
        for (AMSDKMeetingClient user : users) {
            mUsers.add(user);
        }
        updateSpanCount();
        notifyDataSetChanged();
    }

    public void addUser(AMSDKMeetingClient user) {
        Logger.d(LOG_TAG, "addUser:" + user.toString());
        updateUserList(user);
//        notifyItemInserted(mUsers.size() - 1);
        updateSpanCount();
        notifyDataSetChanged();
        mListener.onDataSetChanged();
    }

    public void addUsers(List<AMSDKMeetingClient> users) {
        for (AMSDKMeetingClient user : users) {
            updateUserList(user);
//            notifyItemInserted(mUsers.size() - 1);
        }
        updateSpanCount();
        notifyDataSetChanged();
    }

    private void updateUserList(AMSDKMeetingClient user) {
        for (AMSDKMeetingClient item : mUsers) {
            if (isUserEquals(item, user)) {
                int index = mUsers.indexOf(item);
                mUsers.remove(item);
                mUsers.add(index, user);
                return;
            }
        }
        mUsers.add(user);
    }

    public void removeUserById(String id) {
        Logger.d(LOG_TAG, "removeUserById:" + id);
        int index = getUserIndex(id);
        if (index > -1) {
            mUsers.remove(index);
            notifyItemRemoved(index);
            updateSpanCount();
            notifyDataSetChanged();
            mListener.onDataSetChanged();
        }
    }

    private int getUserIndex(String id) {
        int size = mUsers.size();
        int index = -1;
        for (int i = 0; i < size; i++) {
            AMSDKMeetingClient item = mUsers.get(i);
            if (TextUtils.equals(id, item.getUuid())) {
                index = i;
                break;
            }
        }
        return index;
    }

    public boolean hasUser(String id) {
        return getUserIndex(id) > 0;
    }

    public void removeUsers(List<String> ids) {
        for (String id : ids) {
            int index = getUserIndex(id);
            if (index > -1) {
                mUsers.remove(index);
            }
        }
        updateSpanCount();
        notifyDataSetChanged();
        mListener.onDataSetChanged();
    }

    private void updateSpanCount() {
        adjustPlaceHolders();

        if (mPageSize != LARGE_PAGE_SIZE && mUsers.size() > NORMAL_PAGE_SIZE) {
            mLayoutManager.setSpanCount(LARGE_PAGE_SPAN_COUNT);
            mItemDecoration.setGridSize(LARGE_PAGE_SPAN_COUNT);
            mLayoutManager.setPageSize(LARGE_PAGE_SPAN_COUNT, LARGE_PAGE_SPAN_COUNT);
            mPageSize = LARGE_PAGE_SIZE;
        } else if (mPageSize != NORMAL_PAGE_SIZE && mUsers.size() > SMALL_PAGE_SIZE
                && mUsers.size() <= NORMAL_PAGE_SIZE) {
            mLayoutManager.setSpanCount(NORMAL_PAGE_SPAN_COUNT);
            mItemDecoration.setGridSize(NORMAL_PAGE_SPAN_COUNT);
            mLayoutManager.setPageSize(NORMAL_PAGE_SPAN_COUNT, NORMAL_PAGE_SPAN_COUNT);
            mPageSize = NORMAL_PAGE_SIZE;
        } else if (mPageSize != SMALL_PAGE_SIZE && mUsers.size() == 1) {
            mLayoutManager.setSpanCount(SMALL_PAGE_SPAN_COUNT);
            mItemDecoration.setGridSize(SMALL_PAGE_SPAN_COUNT);
            mLayoutManager.setPageSize(SMALL_PAGE_SPAN_COUNT, SMALL_PAGE_SPAN_COUNT);
            mPageSize = SMALL_PAGE_SIZE;
        } else {
            mLayoutManager.update();
        }
    }

    private void adjustPlaceHolders() {
        int size = getItemCount();
        int actualSize = calculateItemSize(mUsers.size());
        if (actualSize > size) {
            for (int i = 0; i < actualSize - size; i++) {
                mPlaceHolders.add(Boolean.FALSE);
            }
            notifyItemRangeInserted(size, actualSize - size);
        } else {
            for (int i = 0; i < size - actualSize && mPlaceHolders.size() > 0; i++) {
                mPlaceHolders.remove(mPlaceHolders.size() - 1);
            }
            notifyItemRangeRemoved(actualSize - 1, size - actualSize);

        }
    }

    public void updateUser(AMSDKMeetingClient user) {
        int index = getUserIndex(user.getUuid());
        if (index >= 0) {
            mUsers.set(index, user);
            notifyUserChanged(index);
        }
    }

    private void notifyUserChanged(int index) {
        notifyItemChanged(index);
        int firstPosition = mLayoutManager.findFirstVisibleItemPosition();
        int lastPosition = mLayoutManager.findLastVisibleItemPosition();
        if (index >= firstPosition && index <= lastPosition) {
            Logger.d(LOG_TAG, "notify User change visible position " + index);
            mListener.onVisibleRenderChanged(index, true);
        } else {
            Logger.d(LOG_TAG, "notify User change not visible position " + index);
        }
    }

    public void flipUserById(String userId) {
        int index = getUserIndex(userId);
        if (index >= 0) {
            AMSDKMeetingClient userVo = mUsers.get(index);
            notifyUserChanged(index);
        }

    }

    public AMSDKMeetingClient getUser(int pos) {
        if (pos >= 0 && pos < mUsers.size()) {
            return mUsers.get(pos);
        }
        return null;
    }

    public List<String> getUserIdsByPage(int page) {
        List<String> users = new ArrayList<>(mPageSize);
        int len = (page + 1) * mPageSize;
        int size = mUsers.size();
        for (int i = page * mPageSize; i < len && i < size; i++) {
            users.add(mUsers.get(i).getUuid());
        }
        return users;
    }


    private static boolean isUserEquals(AMSDKMeetingClient u1, AMSDKMeetingClient u2) {
        if (u1 == u2) {
            return true;
        }
        if (u1 == null || u2 == null) {
            return false;
        }

        return TextUtils.equals(u1.getUuid(), u2.getUuid()) &&
                TextUtils.equals(u1.getName(), u2.getName());
    }

    static class RenderHolder extends RecyclerView.ViewHolder implements View.OnClickListener {

        private UserRenderView renderView;
        private RenderListener mListener;

        public RenderHolder(View itemView, AMSDKEglBase elgBase, RenderListener listener) {
            super(itemView);
            renderView = new UserRenderView((ConstraintLayout) itemView, elgBase);
            mListener = listener;
        }

        public void setUp(AMSDKMeetingClient client) {
            renderView.setClient(Platform.appContext, client);
        }

        public void release(){
            renderView.release();
        }

        @Override
        public void onClick(View view) {
            mListener.onItemClicked(getAdapterPosition());
        }
    }


}
