package com.keao.alimeeting.detail

import android.app.Activity
import android.os.Build
import android.os.Bundle
import android.support.v7.widget.SimpleItemAnimator
import android.text.TextUtils
import android.util.Log
import android.view.View
import com.alibaba.fastjson.JSONArray
import com.alibaba.fastjson.JSONObject
import com.keao.alimeeting.utils.scheduleMain
import com.keao.alimeeting.utils.showToast
import com.keao.alimeeting.widget.PageGridLayoutManager
import com.keao.alimeeting.R
import com.aliwork.meeting.api.AMSDKErrorCode
import com.aliwork.meeting.api.AMSDKFinishCode
import com.aliwork.meeting.api.AMSDKMeetingCallBack
import com.aliwork.meeting.api.AMSDKMeetingStatus
import com.aliwork.meeting.api.device.AMSDKDeviceManager
import com.aliwork.meeting.api.member.AMSDKClientListEvent
import com.aliwork.meeting.api.member.AMSDKMeetingClient
import com.aliwork.meeting.api.member.AMSDKStatusEvent
import com.aliwork.meeting.api.render.AMSDKEglBase
import com.keao.alimeeting.utils.Platform
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.activity_video_conference_detail.*
import kotlinx.android.synthetic.main.layout_meeting_detail.*
import kotlinx.android.synthetic.main.layout_meeting_detail.view.*
import kotlinx.android.synthetic.main.layout_meeting_entering.view.*
import java.util.concurrent.TimeUnit

class VideoMeetingDetailActivity : Activity(), View.OnClickListener,
        ConferenceView, AMSDKMeetingCallBack, RenderListAdapter.RenderListener {

    private val TAG = "VideoMeetingDetailPage"

    override fun onItemClicked(position: Int) {

    }

    override fun onVisibleRenderChanged(position: Int, visible: Boolean) {
    }

    override fun onDataSetChanged() {
    }

    override fun onStreamStatusChange(client: AMSDKMeetingClient, streamActive: Boolean) {
        Log.d(TAG, "onStreamStatusChange ${client.getName()}")
    }

    override fun onMeetingFinish(code: AMSDKFinishCode, reason: String?) {
        Log.d(TAG, "onMeetingFinish $code $reason")
        showToast(this, "会议结束:$code $reason")
        finish()
    }

    override fun onChannelMessage(msg: String) {
        Log.d(TAG, "onChannelMessage $msg")
    }

    override fun onError(code: AMSDKErrorCode, reason: String?) {
        showToast(this, "onError:$code $reason")
        finish()
    }

    override fun onClientListChange(list: List<AMSDKMeetingClient>, event: AMSDKClientListEvent) {
        when (event) {
            AMSDKClientListEvent.EVENT_INIT -> {
                list.forEach {
                    renderListAdapter?.addUser(it)
                }
            }
//            AMSDKClientListEvent.EVENT_ADD -> {
//                list.forEach {
//                    renderListAdapter?.addUser(it)
//                }
//            }
            AMSDKClientListEvent.EVENT_REMOVE -> {
                list.forEach {
                    renderListAdapter?.removeUserById(it.getUuid())
                }
            }
        }
    }

    override fun onMeetingStatusChange(status: AMSDKMeetingStatus) {
        Log.d(TAG, "onMeetingStatusChange $status")
    }

    override fun onClientStatusChange(client: AMSDKMeetingClient, event: AMSDKStatusEvent) {
        Log.d(
                TAG,
                "onUserStatusChange ${client.getUuid()} ${client.getName()} " +
                        "event:$event ${client.isOnline()} ${client.isMaster()} " +
                        "avatarUrl:${client.getAttribute("avatarUrl")}  deptName:${client.getAttribute("deptName")} jobDesc:${client.getAttribute(
                                "jobDesc"
                        )}"
        )
        if (client.isPublisher()) {
            updateSelfStatus(client)
        }
        when (event) {
            AMSDKStatusEvent.STATUS_ONLINE -> {
                if (client.isOnline()) {
                    if (confPresenter.isInBlackVideoMode()) {
                        client.enableVideo(false, null)
                    } else {
                        confPresenter.enableHDVideo(client, true)
                    }
                    renderListAdapter?.addUser(client)
                } else {
                    if (TextUtils.equals(mainSpeaker?.getUuid(), client.getUuid())) {
                        txtCurrentMemberId.text = "主讲人:无"
                    }
                    if (client.isPublisher()) {
                        showToast(this@VideoMeetingDetailActivity, "被踢出会议了")
                        confPresenter.triggerHangUp()
                        finish()
                    } else {
                        renderListAdapter?.removeUserById(client.getUuid())
                    }
                }
            }
            AMSDKStatusEvent.STATUS_MAIN_SPEAKER -> {
                //如果需要显示主讲人，请保存该值，在ONLINE消息中判断是否是主讲人，
                // 如果主讲人OFFLINE 不会再发送当前主讲人改变的STATUS_MAIN_SPEAKER 事件，
                // 只有等到有下一个主讲人时才会再次触发 STATUS_MAIN_SPEAKER
                mainSpeaker = client
                txtCurrentMemberId.text = "主讲人:${client.getName()}"
                renderListAdapter?.updateUser(client)
            }
            else -> {
                renderListAdapter?.updateUser(client)
            }
        }
    }

    private fun updateSelfStatus(client: AMSDKMeetingClient?) {

        val speaker = AMSDKDeviceManager.getManager(this).isSpeakerEnabled()

        if (speaker) {
            btnSpeakOn.text = "关闭免提"
        } else {
            btnSpeakOn.text = "开启免提"
        }


        if (client == null) {
            return
        }
        if (client.isAudioOn()) {
            btnAMute.text = "静音"
        } else {
            btnAMute.text = "取消静音"
        }
    }

    override fun onMeetingJoined() {
        confPresenter.startMediaStatus()
        scheduleMain({
            enteringLayout.visibility = View.GONE
        })

        timerDisposable = Observable.interval(0, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    joinTime.setText("入会时间:${it}S")
                }
    }

    val confPresenter = ConferencePresenter()

    private var renderListAdapter: RenderListAdapter? = null
    private var mLayoutManager: PageGridLayoutManager? = null
    private var mainSpeaker: AMSDKMeetingClient? = null
    private var timerDisposable: Disposable? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        Platform.initData(this);
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_video_conference_detail)

        detailLayout.keepScreenOn = true
        confPresenter.onViewAttached(this)

        val userId = intent.getStringExtra("userId")
        val memberUuid = intent.getStringExtra("memberUuid")
        val meetingId = intent.getStringExtra("meetingId")
        val wsToken = intent.getStringExtra("wsToken")
        val wsDomian = intent.getStringExtra("wsDomain")
        val clientAppId = intent.getStringExtra("clientAppId")

        if (TextUtils.isEmpty(meetingId) || TextUtils.isEmpty(memberUuid)) {
            showToast(this@VideoMeetingDetailActivity, "获取用户Token异常")
            finish()
            return
        }

        txtCurrentMemberId.text = "当前用户Id:$memberUuid"
        confPresenter.initMeetingInfo(intent, userId, meetingId, memberUuid, clientAppId)
        onMeetingInfoGet(wsToken, wsDomian)
    }

    private fun onMeetingInfoGet(wsToken: String, wsDomian: String) {
        val eglBase = AMSDKEglBase()
        mLayoutManager = PageGridLayoutManager(this, 2)
        renderListAdapter = RenderListAdapter(this, this, eglBase, mLayoutManager, renderList)
        renderList.adapter = renderListAdapter
        renderList.layoutManager = mLayoutManager

        //remove animator for change item
        val itemAnimator = renderList.itemAnimator
        if (itemAnimator is SimpleItemAnimator) {
            itemAnimator.supportsChangeAnimations = false
        }

        confPresenter.start(this, wsToken, wsDomian, eglBase, this)

        detailLayout.btnAMute.setOnClickListener(this)
        detailLayout.btnSpeakOn.setOnClickListener(this)
        detailLayout.btnBlackVideo.setOnClickListener(this)
        detailLayout.btnOpenCamera.setOnClickListener(this)
        detailLayout.btnHangUp.setOnClickListener(this)
        detailLayout.btnSwitchCamera.setOnClickListener(this)
        detailLayout.btnHangUpAll.setOnClickListener(this)
        enteringLayout.imgHangUp.setOnClickListener(this)

        Log.d(
                "DeviceDetail",
                "deviceInfo, model:${Build.MODEL} ${Build.BRAND}  ${Build.BOARD} ${Build.DEVICE}  ${Build.HARDWARE} ${Build.MANUFACTURER}  ${Build.PRODUCT}"
        )
    }


    override fun onMeetingFinished() {
        finish()
    }

    override fun onBackPressed() {
        super.onBackPressed()
        confPresenter.triggerHangUp()
        finish()
    }

    private var isVideoAllEnabled = true
    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.btnAMute -> {
                confPresenter.triggerAMute()
            }
            R.id.btnSpeakOn -> {
                confPresenter.triggerSpeakOn()
                updateSelfStatus(null)
            }
            R.id.btnBlackVideo -> {
                confPresenter.triggerBlackVideo(isVideoAllEnabled)
                isVideoAllEnabled = !isVideoAllEnabled
            }
            R.id.btnOpenCamera -> {
                confPresenter.triggerOpenCamera()
            }
            R.id.btnHangUp, R.id.imgHangUp -> {
                confPresenter.triggerHangUp()
                finish()
            }
            R.id.btnHangUpAll -> {
                confPresenter.triggerHangUpAll()
                finish()
            }
            R.id.btnSwitchCamera -> {
                confPresenter.triggerSwitchCamera()
            }
        }
    }

    private fun findMediaStatus(parent: JSONArray?, clientUuid: String): JSONObject? {
        if (parent == null) {
            return null
        }
        val size = parent.size
        for (i in 0 until size) {
            val child = parent.getJSONObject(i)
            if (child != null) {
                val trackId = child.getString("trackId")
                if (trackId != null && (trackId.startsWith(clientUuid) || trackId.equals(
                                clientUuid,
                                ignoreCase = true
                        ))
                ) {
                    return child
                }
            }
        }
        return null
    }

    override fun updateMediaStatus(result: JSONObject?) {

        val video = result?.getJSONArray("video")
        val audio = result?.getJSONArray("audio")

        val itemCount = renderListAdapter?.getItemCount() ?: return
        for (i in 0 until itemCount) {
            val client = renderListAdapter?.getUser(i)
            if (client != null) {
                val clientUuid = client.getAttribute("streamUuid")
                val params = JSONObject()
                if (video != null) {
                    val subVideo = findMediaStatus(video, clientUuid!!)
                    if (subVideo != null) {
                        val videoParams = JSONObject()
                        if (client.isPublisher()) {
                            videoParams["sentFR"] = subVideo!!.getString("sentFR")
                            videoParams["sentH"] = subVideo!!.getString("sentH")
                            videoParams["sentW"] = subVideo!!.getString("sentW")

                            val videoBwe = result.getJSONObject("VideoBwe")
                            if (videoBwe != null) {
                                videoParams["actualEncBr"] = videoBwe.getString("actualEncBr")
                            }
                        } else {
                            videoParams["recvFR"] = subVideo!!.getString("recvFR")
                            videoParams["recvH"] = subVideo!!.getString("recvH")
                            videoParams["recvW"] = subVideo!!.getString("recvW")
                            videoParams["renDelayMs"] = subVideo!!.getString("renDelayMs")
                        }
                        params["video"] = videoParams
                    }
                }
                if (audio != null) {
                    val subAudio = findMediaStatus(audio, clientUuid!!)
                    if (subAudio != null) {
                        val audioParams = JSONObject()
                        audioParams["curDelayMs"] = subAudio!!.getString("curDelayMs")
                        params["audio"] = audioParams
                    }
                }
                client.setExtraData(params)
                renderListAdapter?.updateUser(client)
            }
        }
    }

    override fun postRunnable(runnable: Runnable) {
        btnHangUp.post(runnable)
    }

    override fun onDestroy() {
        confPresenter.onDestroyed()
        super.onDestroy()
    }

}


