package com.keao.alimeeting.utils

import android.content.Context
import android.util.Log
import android.widget.Toast
import com.alibaba.fastjson.JSON
import com.keao.alimeeting.utils.Platform.appContext
import io.reactivex.Scheduler
import io.reactivex.android.schedulers.AndroidSchedulers
import org.json.JSONException
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStream
import java.io.InputStreamReader
import java.util.concurrent.TimeUnit

object Platform {

    @JvmStatic
    fun initData(context: Context) {
        appContext = context.applicationContext
    }

    lateinit var appContext: Context

    @JvmStatic
    fun readConfig(assetFileName: String): List<EnvParams>? {
        val list = arrayListOf<EnvParams>()
        try {
            val configFile = Platform.appContext.assets.open(assetFileName)
            val jsonStr = readFile(configFile)
            JSON.parseArray(jsonStr)?.forEach {
               val item =  JSON.parseObject(it.toString(), EnvParams::class.java)
                Log.d("Platform", "readConfig item:${JSON.toJSONString(item)}")
                list.add(item)
            }
            return list
        } catch (e: JSONException) {
            e.printStackTrace()
        }
        return list
    }


    @JvmStatic
    @Throws(IOException::class)
    private fun readFile(configFile: InputStream): String {
        var reader: BufferedReader? = null

        try {
            reader = BufferedReader(InputStreamReader(configFile))
            val sb = StringBuilder()

            var line: String? = reader.readLine()
            while ((line) != null) {
                sb.append(line).append("\n")
                line = reader.readLine()
            }
            return sb.toString()
        } finally {
            reader?.close()
        }
    }
}


fun dipToPx(dp: Int): Int {
    val scale = appContext.getResources().getDisplayMetrics().density
    return (dp.toFloat() * scale + 0.5f).toInt()
}

fun showToast(context: Context, msg: String) {
    Toast.makeText(context, msg, Toast.LENGTH_LONG).show()
}

fun scheduleMain(r: () -> Unit, delayMillis: Long = 0) {
    return scheduleRunnable(AndroidSchedulers.mainThread(), r, delayMillis)
}

fun scheduleRunnable(scheduler: Scheduler, r: () -> Unit, delayMillis: Long = 0) {
    val worker = scheduler.createWorker()
    worker.schedule({
        r.invoke()
        worker.dispose()
    }, delayMillis, TimeUnit.MILLISECONDS)
}
