package com.keao.alimeeting.widget

import android.annotation.SuppressLint
import android.content.Context
import android.support.constraint.ConstraintLayout
import android.support.constraint.ConstraintSet
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import com.alibaba.fastjson.JSONObject
import com.keao.alimeeting.R
import com.keao.alimeeting.utils.Logger
import com.keao.alimeeting.utils.Platform
import com.keao.alimeeting.utils.scheduleMain
import com.aliwork.meeting.api.AMSDKActionCallBack
import com.aliwork.meeting.api.AMSDKNetworkStatus
import com.aliwork.meeting.api.member.AMSDKMeetingClient
import com.aliwork.meeting.api.render.AMSDKEglBase
import com.aliwork.meeting.api.render.AMSDKRendererEvents
import com.aliwork.meeting.api.render.AMSDKScalingType
import com.aliwork.meeting.api.render.AMSDKTextureRenderView


class UserRenderView(private var renderRootView: ConstraintLayout, var eglBase: AMSDKEglBase) :
    View.OnClickListener, AMSDKRendererEvents {

    override fun onClick(v: View?) {
        if (curClient == null) {
            Toast.makeText(Platform.appContext, "Client is Null", Toast.LENGTH_LONG).show()
        }
        actionLayout.visibility = View.GONE
        when (v?.id) {
            R.id.btnHangUp -> {
                curClient?.hangup(object:AMSDKActionCallBack{
                    override fun onSuccess() {
                        Log.d(LOG_TAG, "hangup ${curClient?.getName()} success")
                    }

                    override fun onFailed(errorMsg: String?) {
                        Log.d(LOG_TAG, "hangup ${curClient?.getName()} failed, $errorMsg")
                    }

                })
            }
            R.id.btnMuteVideo -> {
                curClient?.muteVideo(curClient?.isVideoOn() ?: false, null)
            }
            R.id.btnMuteAudio -> {
                curClient?.muteAudio(curClient?.isAudioOn() ?: false, null)
            }
        }
    }

    companion object {
        const val LOG_TAG = "userRenderView"
    }

    private val userAvatar: FrameLayout = renderRootView.findViewById(R.id.userAvatar)
    private val userTalkingStatus: ImageView = renderRootView.findViewById(R.id.userTalkingStatus)
    private val userStatus: TextView = renderRootView.findViewById(R.id.userStatus)
    private val userDetails: TextView = renderRootView.findViewById(R.id.userDetails)

    private val userName: TextView = renderRootView.findViewById(R.id.userName)
    private val debugInfo: TextView = renderRootView.findViewById(R.id.debugInfo)
    var renderView: AMSDKTextureRenderView? = null
    var userId: String? = null
    private var curClient: AMSDKMeetingClient? = null

    private val actionLayout: ViewGroup = renderRootView.findViewById(R.id.actionLayout)

    private var mIsOnceRendered = false

    init {
        renderRootView.apply {
            setOnClickListener {
                if (curClient?.isPublisher() == true) {
                    return@setOnClickListener
                }
                if (actionLayout.visibility == View.VISIBLE) {
                    actionLayout.visibility = View.GONE
                } else {
                    actionLayout.visibility = View.VISIBLE
                }
            }
            findViewById<View>(R.id.btnHangUp).setOnClickListener(this@UserRenderView)
            findViewById<View>(R.id.btnMuteVideo).setOnClickListener(this@UserRenderView)
            findViewById<View>(R.id.btnMuteAudio).setOnClickListener(this@UserRenderView)
        }
    }

    fun setClient(context: Context, client: AMSDKMeetingClient) {
        if (curClient == null) {
            checkAndInitRenderView(context)
            client.addRenderView(renderView)
        } else {
            if (client.isVideoOn() && client.isStreamReady()) {
                checkAndInitRenderView(context)
                client.addRenderView(renderView)
            }
            if (!TextUtils.equals(curClient?.getUuid(), client.getUuid())) {
                //非同一个用户的RenderView，检查是否可以服用renderView
                curClient?.removeRenderView(renderView)
            }
        }
        this.curClient = client
        userId = client.getUuid()
        updateStatus(client)
    }

    private fun checkAndInitRenderView(context: Context) {
        if (renderView == null || renderView?.isReleased == true) {
            renderView = AMSDKTextureRenderView(context).apply {
                //添加AMSDKRendererEvents，接收渲染回调
                this.init(eglBase, this@UserRenderView)

                this.setScalingType(
                    AMSDKScalingType.SCALE_ASPECT_FILL,
                    AMSDKScalingType.SCALE_ASPECT_BALANCED
                )
                this.id = R.id.conference_texture_view
                val layoutParams = ConstraintLayout.LayoutParams(
                    ConstraintLayout.LayoutParams.WRAP_CONTENT,
                    ConstraintLayout.LayoutParams.WRAP_CONTENT
                )
                renderRootView.addView(this, 0, layoutParams)
                val set = ConstraintSet()
                set.connect(
                    this.id,
                    ConstraintSet.START,
                    ConstraintSet.PARENT_ID,
                    ConstraintSet.START
                )
                set.connect(this.id, ConstraintSet.END, ConstraintSet.PARENT_ID, ConstraintSet.END)
                set.connect(this.id, ConstraintSet.TOP, ConstraintSet.PARENT_ID, ConstraintSet.TOP)
                set.connect(
                    this.id,
                    ConstraintSet.BOTTOM,
                    ConstraintSet.PARENT_ID,
                    ConstraintSet.BOTTOM
                )
                set.constrainHeight(this.id, ConstraintSet.WRAP_CONTENT)
                set.constrainWidth(this.id, ConstraintSet.WRAP_CONTENT)
                set.applyTo(renderRootView)
            }
        }
    }

    @SuppressLint("SetTextI18n")
    private fun updateStatus(client: AMSDKMeetingClient) {
        userName.text = if (client.isPublisher()) "我自己:${client.getName()}" else client.getName()
        Logger.d(
            LOG_TAG,
            "AMSDKMemberManager updateStatus ${client.getName()} ${client.isPublisher()} " +
                    "${client.getClientType()} ${client.isStreamReady()} ${client.isVideoOn()}"
        )

        if (client.isStreamReady() && client.isVideoOn()) {
            //这里暂时不消失，防止出现黑屏
            if (mIsOnceRendered) {
            userAvatar.visibility = View.GONE
            }
            renderView?.visibility = View.VISIBLE
        } else {

            userAvatar.visibility = View.VISIBLE
            renderView?.visibility = View.GONE
        }

        renderView?.setScalingType(
            AMSDKScalingType.SCALE_ASPECT_BALANCED,
            AMSDKScalingType.SCALE_ASPECT_FIT
        )

        val network = when (client.getNetworkStatus()) {
            AMSDKNetworkStatus.WEAK -> {
                "信号：弱网,${client.getNetworkQuality().ordinal}"
            }
            AMSDKNetworkStatus.NORMAL -> {
                "信号：正常,${client.getNetworkQuality().ordinal}"
            }
            AMSDKNetworkStatus.DISCONNECT -> {
                "信号：已断网"
            }
        }
        val video = if (client.isVideoOn()) "开" else "关"
        val audio = if (client.isAudioOn()) "开" else "关"

        val streamReady = if (client.isStreamReady()) "视频流：开" else "视频流：关"

        val talking = if (client.isTalking()) "正在说话" else "没有说话"
        val ringing = if (client.isRinging()) "正在振铃" else "没有振铃"
        val mainSpeaker = if (client.isMainSpeaker()) "主讲人: 是" else "主讲人: 否"

        userDetails.text =
            "$network \n 视频:$video \n $streamReady \n 音频:$audio \n $talking \n $ringing \n $mainSpeaker"




        if (client.isOnline()) {
            if (client.isTalking()) {
                userTalkingStatus.setImageResource(R.drawable.status_talking)
            } else if (client.isAudioOn()) {
                userTalkingStatus.setImageResource(R.drawable.status_unmute)
            } else {
                userTalkingStatus.setImageResource(R.drawable.status_mute)
            }

            userStatus.visibility = View.GONE
            userTalkingStatus.visibility = View.VISIBLE
            debugInfo.visibility = View.VISIBLE
            userDetails.visibility = View.VISIBLE
        } else {
            if (client.isRinging()) {
                userStatus.text = "等待对方入会"
            } else {
                userStatus.text = ""
            }
            userStatus.visibility = View.VISIBLE
            userDetails.visibility = View.GONE
            debugInfo.visibility = View.GONE
            userTalkingStatus.visibility = View.GONE
            return
        }

        val extraData = client.getExtraData()
        if (extraData is JSONObject) {
            try {
                val builder = StringBuilder()
                var params: JSONObject? = extraData.getJSONObject("video")
                if (params != null) {
                    builder.append("video:\n")
                    for (key in params.keys) {
                        builder.append("     ").append(key).append(":")
                            .append(params.getString(key)).append("\n")
                    }
                }
                params = extraData.getJSONObject("audio")
                if (params != null) {
                    builder.append("audio:\n")
                    for (key in params.keys) {
                        builder.append("     ").append(key).append(":")
                            .append(params.getString(key)).append("\n")
                    }
                }

                debugInfo.text = builder
            } catch (e: Exception) {
                e.printStackTrace()
            }

        } else {
            debugInfo.text = ""
        }
    }

    override fun onFirstFrameRendered() {
        scheduleMain({
            mIsOnceRendered = true
            userAvatar.visibility = View.GONE
        })
    }

    override fun onFrameResolutionChanged(videoWidth: Int, videoHeight: Int, rotation: Int) {
    }

    fun release() {
        Logger.i(LOG_TAG, "release render view ")
        renderView?.let {
            renderRootView.removeView(it as View)
            renderView = null
        }
    }

}